#include "stdlib.h"
#include "queue.h"

/* implementation of queue */
typedef struct queue_item {
  void *q_item;		/* not responsible for freeing items! */
  struct queue_item *q_next;
} queue_item;

struct queue {
  queue_item *q_front;
  queue_item *q_back;
};

/* create a new empty queue (FIFO) */
queue *new_queue() {
  queue *q = malloc(sizeof(struct queue));
  queue_item *sentinel = malloc (sizeof (struct queue_item));
  if (q == NULL || sentinel == NULL) {
    return NULL;
  } else {
    q->q_front = q->q_back = sentinel;
    return q;
  }
}

/* add element to back of the queue */
bool enqueue (queue *q, void *elem) {
  if (q == NULL) {
    return false;
  } else {
    queue_item *qi = q->q_back;
    qi->q_item = elem;
    qi->q_next = malloc (sizeof (struct queue_item));
    q->q_back = qi->q_next;
    return (qi->q_next != NULL);
  }
}

/* take element from front of queue */
void *dequeue (queue *q) {
  if (is_empty (q)) {
    return NULL;
  } else {
    queue_item *qi = q->q_front;
    void *r = qi->q_item;
    q->q_front = qi->q_next;
    free (qi);
    return r;
  }
}

bool is_empty (queue *q) {
  if (q == NULL) {
    return true;
  } else {
    return (q->q_front == q->q_back);
  }
}

/* must be empty to free */
bool free_queue(queue *q) {
  if (!is_empty (q) || q == NULL) {
    return false;
  } else {
    free (q->q_front);
    free (q);
    return true;
  }
}


/*********************/
  
 
